@@ -22,9 +22,10 @@ from .const import (
     ECOBEE_ENDPOINT_THERMOSTAT,
     ECOBEE_ENDPOINT_TOKEN,
     ECOBEE_REFRESH_TOKEN,
+    ECOBEE_OPTIONS_NOTIFICATIONS,
 )
 from .errors import ExpiredTokenError, InvalidTokenError
-from .util import config_from_file
+from .util import config_from_file, convert_to_bool
 
 
 class Ecobee(object):
@@ -39,6 +40,7 @@ class Ecobee(object):
         self.authorization_code = None
         self.access_token = None
         self.refresh_token = None
+        self.include_notifications = False
 
         if self.config_filename is None and self.config is None:
             _LOGGER.error("No ecobee credentials supplied, unable to continue")
@@ -53,6 +55,8 @@ class Ecobee(object):
                 self.authorization_code = self.config[ECOBEE_AUTHORIZATION_CODE]
             if ECOBEE_REFRESH_TOKEN in self.config:
                 self.refresh_token = self.config[ECOBEE_REFRESH_TOKEN]
+            if ECOBEE_OPTIONS_NOTIFICATIONS in self.config:
+                self.include_notifications = convert_to_bool(self.config[ECOBEE_OPTIONS_NOTIFICATIONS])
         else:
             self._file_based_config = True
 
@@ -67,6 +71,8 @@ class Ecobee(object):
                 self.authorization_code = self.config[ECOBEE_AUTHORIZATION_CODE]
             if ECOBEE_REFRESH_TOKEN in self.config:
                 self.refresh_token = self.config[ECOBEE_REFRESH_TOKEN]
+            if ECOBEE_OPTIONS_NOTIFICATIONS in self.config:
+                self.include_notifications = convert_to_bool(self.config[ECOBEE_OPTIONS_NOTIFICATIONS])
 
     def _write_config(self) -> None:
         """Writes API tokens to a file or self.config if self.file_based_config is False."""
@@ -75,6 +81,7 @@ class Ecobee(object):
         config[ECOBEE_ACCESS_TOKEN] = self.access_token
         config[ECOBEE_REFRESH_TOKEN] = self.refresh_token
         config[ECOBEE_AUTHORIZATION_CODE] = self.authorization_code
+        config[ECOBEE_OPTIONS_NOTIFICATIONS] = str(self.include_notifications)
         if self._file_based_config:
             config_from_file(self.config_filename, config)
         else:
@@ -107,7 +114,8 @@ class Ecobee(object):
                 f"After authorizing, call request_tokens method."
             )
             return True
-        except (KeyError, TypeError):
+        except (KeyError, TypeError) as err:
+            _LOGGER.debug(f"Error obtaining PIN code from ecobee: {err}")
             return False
 
     def request_tokens(self) -> bool:
@@ -132,8 +140,11 @@ class Ecobee(object):
             self.refresh_token = response["refresh_token"]
             self._write_config()
             self.pin = None
+            _LOGGER.debug(f"Obtained tokens from ecobee: access {self.access_token}, "
+                          f"refresh {self.refresh_token}")
             return True
-        except (KeyError, TypeError):
+        except (KeyError, TypeError) as err:
+            _LOGGER.debug(f"Error obtaining tokens from ecobee: {err}")
             return False
 
     def refresh_tokens(self) -> bool:
@@ -157,8 +168,11 @@ class Ecobee(object):
             self.access_token = response["access_token"]
             self.refresh_token = response["refresh_token"]
             self._write_config()
+            _LOGGER.debug(f"Refreshed tokens from ecobee: access {self.access_token}, "
+                          f"refresh {self.refresh_token}")
             return True
-        except (KeyError, TypeError):
+        except (KeyError, TypeError) as err:
+            _LOGGER.debug(f"Error refreshing tokens from ecobee: {err}")
             return False
 
     def get_thermostats(self) -> bool:
@@ -175,6 +189,8 @@ class Ecobee(object):
                 "includeSettings": "true",
             }
         }
+        if self.include_notifications:
+            param_string["selection"]["includeNotificationSettings"] = self.include_notifications
         params = {"json": json.dumps(param_string)}
         log_msg_action = "get thermostats"
 
@@ -195,6 +211,10 @@ class Ecobee(object):
     def get_remote_sensors(self, index: int) -> str:
         """Returns remote sensors from a thermostat based on list index of self.thermostats."""
         return self.thermostats[index]["remoteSensors"]
+    
+    def get_equipment_notifications(self, index: int) -> str:
+        """Returns equipment notifications from a thermostat based on list index of self.thermostats."""
+        return self.thermostats[index]["notificationSettings"]["equipment"]
 
     def update(self) -> bool:
         """Gets new thermostat data from ecobee; wrapper for get_thermostats."""
@@ -271,24 +291,44 @@ class Ecobee(object):
         cool_temp: int,
         heat_temp: int,
         hold_type: str = "nextTransition",
+        hold_hours: str = "2",
     ) -> None:
         """Sets a hold temperature."""
-        body = {
-            "selection": {
-                "selectionType": "thermostats",
-                "selectionMatch": self.thermostats[index]["identifier"],
-            },
-            "functions": [
-                {
-                    "type": "setHold",
-                    "params": {
-                        "holdType": hold_type,
-                        "coolHoldTemp": int(cool_temp * 10),
-                        "heatHoldTemp": int(heat_temp * 10),
-                    },
-                }
-            ],
-        }
+        if hold_type == "holdHours":
+            body = {
+                "selection": {
+                    "selectionType": "thermostats",
+                    "selectionMatch": self.thermostats[index]["identifier"],
+                },
+                "functions": [
+                    {
+                        "type": "setHold",
+                        "params": {
+                            "holdType": hold_type,
+                            "coolHoldTemp": int(cool_temp * 10),
+                            "heatHoldTemp": int(heat_temp * 10),
+                            "holdHours": hold_hours,
+                        },
+                    }
+                ],
+            }
+        else:
+            body = {
+                "selection": {
+                    "selectionType": "thermostats",
+                    "selectionMatch": self.thermostats[index]["identifier"],
+                },
+                "functions": [
+                    {
+                        "type": "setHold",
+                        "params": {
+                            "holdType": hold_type,
+                            "coolHoldTemp": int(cool_temp * 10),
+                            "heatHoldTemp": int(heat_temp * 10),
+                        },
+                    }
+                ],
+            }
         log_msg_action = "set hold temp"
 
         try:
@@ -512,9 +552,15 @@ class Ecobee(object):
             response = requests.request(
                 method, url, headers=headers, params=params, json=body, timeout=ECOBEE_DEFAULT_TIMEOUT
             )
+
+            try:
+                log_msg = response.json()
+            except:
+                log_msg = response.text
             _LOGGER.debug(
-                f"Request response: {response.status_code}: {response.text}"
+                f"Request response: {response.status_code}: {log_msg}"
             )
+            
             response.raise_for_status()
             return response.json()
         except HTTPError:
